"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transform = transform;
exports.transformFromAst = transformFromAst;
exports.registerPlugin = registerPlugin;
exports.registerPlugins = registerPlugins;
exports.registerPreset = registerPreset;
exports.registerPresets = registerPresets;
exports.buildExternalHelpers = exports.availablePresets = exports.availablePlugins = void 0;

var _core = require("@babel/core");

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var isArray = Array.isArray || function (arg) {
  return Object.prototype.toString.call(arg) === '[object Array]';
};
/**
 * Loads the given name (or [name, options] pair) from the given table object
 * holding the available presets or plugins.
 *
 * Returns undefined if the preset or plugin is not available; passes through
 * name unmodified if it (or the first element of the pair) is not a string.
 */


function loadBuiltin(builtinTable, name) {
  if (isArray(name) && typeof name[0] === 'string') {
    if (builtinTable.hasOwnProperty(name[0])) {
      return [builtinTable[name[0]]].concat(name.slice(1));
    }

    return;
  } else if (typeof name === 'string') {
    return builtinTable[name];
  } // Could be an actual preset/plugin module


  return name;
}
/**
 * Parses plugin names and presets from the specified options.
 */


function processOptions(options) {
  // Parse preset names
  var presets = (options.presets || []).map(function (presetName) {
    var preset = loadBuiltin(availablePresets, presetName);

    if (preset) {
      // workaround for babel issue
      // at some point, babel copies the preset, losing the non-enumerable
      // buildPreset key; convert it into an enumerable key.
      if (isArray(preset) && _typeof(preset[0]) === 'object' && preset[0].hasOwnProperty('buildPreset')) {
        preset[0] = _objectSpread({}, preset[0], {
          buildPreset: preset[0].buildPreset
        });
      }
    } else {
      throw new Error("Invalid preset specified in Babel options: \"".concat(presetName, "\""));
    }

    return preset;
  }); // Parse plugin names

  var plugins = (options.plugins || []).map(function (pluginName) {
    var plugin = loadBuiltin(availablePlugins, pluginName);

    if (!plugin) {
      throw new Error("Invalid plugin specified in Babel options: \"".concat(pluginName, "\""));
    }

    return plugin;
  });
  return _objectSpread({
    babelrc: false
  }, options, {
    presets: presets,
    plugins: plugins
  });
}

function transform(code, options) {
  return (0, _core.transform)(code, processOptions(options));
}

function transformFromAst(ast, code, options) {
  return (0, _core.transformFromAst)(ast, code, processOptions(options));
}

var availablePlugins = {};
exports.availablePlugins = availablePlugins;
var availablePresets = {};
exports.availablePresets = availablePresets;
var buildExternalHelpers = _core.buildExternalHelpers;
/**
 * Registers a named plugin for use with Babel.
 */

exports.buildExternalHelpers = buildExternalHelpers;

function registerPlugin(name, plugin) {
  if (availablePlugins.hasOwnProperty(name)) {
    console.warn("A plugin named \"".concat(name, "\" is already registered, it will be overridden"));
  }

  availablePlugins[name] = plugin;
}
/**
 * Registers multiple plugins for use with Babel. `newPlugins` should be an object where the key
 * is the name of the plugin, and the value is the plugin itself.
 */


function registerPlugins(newPlugins) {
  Object.keys(newPlugins).forEach(function (name) {
    return registerPlugin(name, newPlugins[name]);
  });
}
/**
 * Registers a named preset for use with Babel.
 */


function registerPreset(name, preset) {
  if (availablePresets.hasOwnProperty(name)) {
    console.warn("A preset named \"".concat(name, "\" is already registered, it will be overridden"));
  }

  availablePresets[name] = preset;
}
/**
 * Registers multiple presets for use with Babel. `newPresets` should be an object where the key
 * is the name of the preset, and the value is the preset itself.
 */


function registerPresets(newPresets) {
  Object.keys(newPresets).forEach(function (name) {
    return registerPreset(name, newPresets[name]);
  });
}